package de.upb.qvto.test;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EPackageRegistryImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.m2m.internal.qvt.oml.InternalTransformationExecutor;
import org.eclipse.m2m.internal.qvt.oml.compiler.CompiledUnit;
import org.eclipse.m2m.internal.qvt.oml.compiler.ExeXMISerializer;

@SuppressWarnings("restriction")
public class Test {
	
	public static void main(String[] args)
	{
		// Load qvto transformation from file.
		// That transformation can import other transformation files.
		//
		ResourceSet rs = new ResourceSetImpl();
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
				Resource.Factory.Registry.DEFAULT_EXTENSION,
				new XMIResourceFactoryImpl());
		
		File dir = new File(".");
		EPackage rdbmsPackage = (EPackage) rs.getResource(URI.createURI(dir.toURI().toString()+"files/SimpleRDBMS.ecore"), true).getContents().get(0); //$NON-NLS-1$
		EPackage umlPackage = (EPackage) rs.getResource(URI.createURI(dir.toURI().toString()+"files/SimpleUML.ecore"), true).getContents().get(0); //$NON-NLS-1$

		EPackage.Registry registry = new EPackageRegistryImpl(EPackage.Registry.INSTANCE);
		registry.put(rdbmsPackage.getNsURI(), rdbmsPackage);
		registry.put(umlPackage.getNsURI(), umlPackage);
		
		URI transformationURI = URI.createURI(dir.toURI().toString()+"files/uml2rdbms.qvto"); //$NON-NLS-1$
		InternalTransformationExecutor executor = new InternalTransformationExecutor(transformationURI, registry);
		Diagnostic diag = executor.loadTransformation();
		if (diag.getSeverity() == Diagnostic.ERROR || diag.getSeverity() == Diagnostic.CANCEL) {
			System.err.println(diag);
			return;
		}

		// Obtain transformation object.
		// Note that we receive abstract type 'Module' since actual object type 
		// is an instance of 'OperationalTransformation' or 'Library'
		CompiledUnit unit = executor.getUnit();
		org.eclipse.m2m.internal.qvt.oml.expressions.Module module = unit.getModules().get(0);

		// Save compiled transformation
		try {
			ExeXMISerializer.saveUnitXMI(new CompiledUnit[] {unit}, registry);
			// don't forget about imported transformations/libraries
			for (CompiledUnit importUnit : unit.getCompiledImports()) {
		  	ExeXMISerializer.saveUnitXMI(new CompiledUnit[] {importUnit}, registry);
			}
		} catch (IOException e) {
			System.err.println("Parser error: could not save with ExeXMISerializer!");
		}

		// move compiled "qvtox" file to target location
		URI xmiQvtoURI = ExeXMISerializer.toXMIUnitURI(transformationURI);
		File source = new File(xmiQvtoURI.toFileString());
		File target = new File(URI.createFileURI("files/uml2rdbms.qvto.xmi").toFileString());
		if(target.exists()) target.delete();
		try {
			FileUtils.moveFile(source, target);
			System.out.println("Saved "+target.toString());
		} catch (IOException e) {
			System.err.println("Unable to move "+source.toString()+" to "+target.toString()+":\n"+e);
		}
	}
}
